@@ -23,10 +23,11 @@
 #include "utils.h"
 #include "rtpsession_priv.h"
 
+#define SSRC_CHANGED_THRESHOLD 50
 
-void split_and_queue(queue_t *q, int maxrqsz, mblk_t *mp, rtp_header_t *rtp, int *discarded)
+static void queue_packet(queue_t *q, int maxrqsz, mblk_t *mp, rtp_header_t *rtp, int *discarded)
 {
-	mblk_t *mdata,*tmp;
+	mblk_t *tmp;
 	int header_size;
 	*discarded=0;
 	header_size=RTP_FIXED_HEADER_SIZE+ (4*rtp->cc);
@@ -36,13 +37,6 @@ void split_and_queue(queue_t *q, int maxrqsz, mblk_t *mp, rtp_header_t *rtp, int
 		freemsg(mp);
 		return;
 	}
-	/* creates a new mblk_t to be linked with the rtp header*/
-	mdata=dupb(mp);
-	
-	mp->b_wptr=mp->b_rptr+header_size;
-	mdata->b_rptr+=header_size;
-	/* link proto with data */
-	mp->b_cont=mdata;
 	/* and then add the packet to the queue */
 	
 	rtp_putq(q,mp);
@@ -68,9 +62,7 @@ void rtp_session_rtp_parse(RtpSession *session, mblk_t *mp, uint32_t local_str_t
 	RtpStream *rtpstream=&session->rtp;
 	rtp_stats_t *stats=&rtpstream->stats;
 	
-	return_if_fail(mp!=NULL);
-	
-	msgsize=msgdsize(mp);
+	msgsize=mp->b_wptr-mp->b_rptr;
 
 	if (msgsize<RTP_FIXED_HEADER_SIZE){
 		ortp_warning("Packet too small to be a rtp packet (%i)!",msgsize);
@@ -95,9 +87,14 @@ void rtp_session_rtp_parse(RtpSession *session, mblk_t *mp, uint32_t local_str_t
 				rtp_session_dispatch_event(session,ev);
 				return;
 			}
+		}else{
+			/* discard*/
+			ortp_debug("Receiving rtp packet with version number !=2...discarded");
+			stats->bad++;
+			ortp_global_stats.bad++;
+			freemsg(mp);
+			return;
 		}
-		freemsg(mp);
-		return;
 	}
 
 	/* only count non-stun packets. */
@@ -107,15 +104,6 @@ void rtp_session_rtp_parse(RtpSession *session, mblk_t *mp, uint32_t local_str_t
 	stats->hw_recv+=msgsize;
 	session->rtp.hwrcv_since_last_SR++;
 
-	if (rtp->version!=2)
-	{
-		/* discard*/
-		ortp_debug("Receiving rtp packet with version number !=2...discarded");
-		stats->bad++;
-		ortp_global_stats.bad++;
-		freemsg(mp);
-		return;
-	}
 	
 	/* convert all header data from network order to host order */
 	rtp->seq_number=ntohs(rtp->seq_number);
@@ -130,20 +118,64 @@ void rtp_session_rtp_parse(RtpSession *session, mblk_t *mp, uint32_t local_str_t
 		return;
 	}
 
+#ifndef PERF
 	/* Write down the last RTP/RTCP packet reception time. */
 	gettimeofday(&session->last_recv_time, NULL);
+#endif
 
 	for (i=0;i<rtp->cc;i++)
 		rtp->csrc[i]=ntohl(rtp->csrc[i]);
-	if (session->recv_ssrc!=0)
-	{
+	/*the goal of the following code is to lock on an incoming SSRC to avoid
+	receiving "mixed streams"*/
+	if (session->ssrc_set){
 		/*the ssrc is set, so we must check it */
-		if (session->recv_ssrc!=rtp->ssrc){
-			/*ortp_debug("rtp_parse: bad ssrc - %i",rtp->ssrc);*/
-			session->recv_ssrc=rtp->ssrc;
-			rtp_signal_table_emit(&session->on_ssrc_changed);
+		if (session->rcv.ssrc!=rtp->ssrc){
+			if (session->inc_ssrc_candidate==rtp->ssrc){
+				session->inc_same_ssrc_count++;
+			}else{
+				session->inc_same_ssrc_count=0;
+				session->inc_ssrc_candidate=rtp->ssrc;
+			}
+			if (session->inc_same_ssrc_count>SSRC_CHANGED_THRESHOLD){
+
+				/* store the sender rtp address to do symmetric RTP */
+				if (!session->use_connect){
+					if (session->rtp.socket>0 && session->symmetric_rtp){
+						/* store the sender rtp address to do symmetric RTP */
+						memcpy(&session->rtp.rem_addr,addr,addrlen);
+						session->rtp.rem_addrlen=addrlen;
+					}
+				}
+
+				session->rcv.ssrc=rtp->ssrc;
+				rtp_signal_table_emit(&session->on_ssrc_changed);
+			}else{
+				/*discard the packet*/
+				ortp_debug("Receiving packet with unknown ssrc.");
+				stats->bad++;
+				ortp_global_stats.bad++;
+				freemsg(mp);
+				return;
+			}
 		}
-	}else session->recv_ssrc=rtp->ssrc;
+    else{
+			/* The SSRC change must not happen if we still receive
+			ssrc from the initial source. */
+			session->inc_same_ssrc_count=0;
+		}
+
+	}else{
+		session->ssrc_set=TRUE;
+		session->rcv.ssrc=rtp->ssrc;
+
+		if (!session->use_connect){
+			if (session->rtp.socket>0 && session->symmetric_rtp){
+				/* store the sender rtp address to do symmetric RTP */
+				memcpy(&session->rtp.rem_addr,addr,addrlen);
+				session->rtp.rem_addrlen=addrlen;
+			}
+		}
+	}
 	
 	/* update some statistics */
 	{
@@ -158,8 +190,8 @@ void rtp_session_rtp_parse(RtpSession *session, mblk_t *mp, uint32_t local_str_t
 	}
 	
 	/* check for possible telephone events */
-	if (rtp->paytype==session->telephone_events_pt){
-		split_and_queue(&session->rtp.tev_rq,session->rtp.max_rq_size,mp,rtp,&i);
+	if (rtp->paytype==session->rcv.telephone_events_pt){
+		queue_packet(&session->rtp.tev_rq,session->rtp.max_rq_size,mp,rtp,&i);
 		stats->discarded+=i;
 		ortp_global_stats.discarded+=i;
 		return;
@@ -171,14 +203,9 @@ void rtp_session_rtp_parse(RtpSession *session, mblk_t *mp, uint32_t local_str_t
 		rtp_session_update_payload_type(session,rtp->paytype);
 	}
 	
+	jitter_control_new_packet(&session->rtp.jittctl,rtp->timestamp,local_str_ts);
+
 	if (session->flags & RTP_SESSION_FIRST_PACKET_DELIVERED) {
-		int32_t slide=0;
-		int32_t safe_delay=0;
-		jitter_control_new_packet(&session->rtp.jittctl,rtp->timestamp,local_str_ts,&slide,&safe_delay);
-		
-		session->rtp.rcv_diff_ts=session->rtp.hwrcv_diff_ts + slide - safe_delay;
-		ortp_debug("  rcv_diff_ts=%i", session->rtp.rcv_diff_ts);
-		
 		/* detect timestamp important jumps in the future, to workaround stupid rtp senders */
 		if (RTP_TIMESTAMP_IS_NEWER_THAN(rtp->timestamp,session->rtp.rcv_last_ts+session->rtp.ts_jump)){
 			ortp_debug("rtp_parse: timestamp jump ?");
@@ -201,10 +228,10 @@ void rtp_session_rtp_parse(RtpSession *session, mblk_t *mp, uint32_t local_str_t
 			ortp_global_stats.outoftime++;
 			return;
 		}
-		
 	}
 	
-	split_and_queue(&session->rtp.rq,session->rtp.max_rq_size,mp,rtp,&i);
+	queue_packet(&session->rtp.rq,session->rtp.max_rq_size,mp,rtp,&i);
 	stats->discarded+=i;
 	ortp_global_stats.discarded+=i;
 }
+
